package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element;

import java.util.HashMap;
import java.io.File;

public class PageAction extends Action {

    public static final String SHORT_ACTION_NAME = "p";
    public static final String HOME_PAGE = "home";
    public static final String ABOUT_PAGE = "about";

    public Element process (Element message) {
	
	Element request = (Element) GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);
	// the page name is the subaction
	String page_name = request.getAttribute(GSXML.SUBACTION_ATT);
	if (page_name.equals("")) { // if no page specified, assume home page
	    page_name = HOME_PAGE;
	}
	
	// create the return page tree
	Element page = doc_.createElement(GSXML.PAGE_ELEM);
	page.setAttribute(GSXML.LANG_ATT, request.getAttribute(GSXML.LANG_ATT));
	// add the page extra stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.PAGE_EXTRA_ELEM), true));
	// add the cgi args request
	Element page_request = GSXML.duplicateWithNewName(doc_, request, "pageRequest", true);
	page.appendChild(page_request);

	if (page_name.equals(HOME_PAGE)) {
	    return homePage(page, request);
	} else if (page_name.equals(ABOUT_PAGE)) {
	    return aboutPage(page, request);
	} 
	
	System.err.println("PageAction Error: unknown page specified!");
	return unknownPage(page, request);
	
    }


    protected Element homePage(Element page, Element orig_message) {
	
	String lang = page.getAttribute(GSXML.LANG_ATT);
	// first, get the message router info
	Element coll_list_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element coll_list_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE, "", lang);
	coll_list_message.appendChild(coll_list_request);
	Element coll_list_response = (Element)mr_.process(coll_list_message);
	if (coll_list_response==null) {
	    System.err.println("PageAction Error: couldn't query the message router!");
	    return null;
	}
	
	// second, get the metadata for each collection - we only want specific
	// elements but for now, we'll just get it all
	NodeList colls = coll_list_response.getElementsByTagName(GSXML.COLLECTION_ELEM);
	// we will send all the requests in a single message
	Element metadata_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	for (int i=0; i<colls.getLength(); i++) {
	    Element c = (Element)colls.item(i);
	    String name = c.getAttribute(GSXML.NAME_ATT);	    
	    
	    Element metadata_request = GSXML.createBasicRequest(doc_,  GSXML.REQUEST_TYPE_DESCRIBE, name, lang);
	    metadata_message.appendChild(metadata_request);
	}

	Element metadata_response = (Element)mr_.process(metadata_message);

	NodeList coll_responses = metadata_response.getElementsByTagName(GSXML.RESPONSE_ELEM);
	// check that have same number of responses as collections
	if (colls.getLength() != coll_responses.getLength()) {
	    System.err.println("PageAction Error: didn't get a response for each collection - somethings gone wrong!");
	    // for now, dont use the metadata
	} else {
	    for (int i=0; i<colls.getLength(); i++) {
		Element c1 = (Element)colls.item(i);
		Element c2 = (Element)coll_responses.item(i);
		if (c1.getAttribute(GSXML.NAME_ATT).equals(c2.getAttribute(GSXML.FROM_ATT))) {
		    // add the metadata to the original response   
		    GSXML.mergeMetadataLists(c1, GSXML.getChildByTagName(c2, GSXML.COLLECTION_ELEM)); // add the metadata to the original response   
		} else {
		    System.err.println("PageAction Error: response does not correspond to request!");
		} 
		
	    }
	}

	//now the full response is in coll_list_response

	// add it in to the page xml tree
	Element response = (Element) GSXML.getChildByTagName(coll_list_response, GSXML.RESPONSE_ELEM);
	Element page_response = GSXML.duplicateWithNewName(doc_, response, "pageResponse", false);
	page.appendChild(page_response);

	String stylesheet = GSFile.stylesheetFile(config_, "home.xsl");
	Document style_doc = converter_.getDOM(new File(stylesheet));
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	return (Element)transformer_.transform(style_doc, page);    	
	
    } // homePage

    protected Element aboutPage(Element page, Element request) {

	// extract the params from the cgi-request, 
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_paramList, false);
	
	String coll_name = (String)params.get(GSCGI.COLLECTION_ARG);
	if (coll_name == null || coll_name.equals("")) {
	    System.err.println("PageAction Error: about page requested with no collection or cluster specified!");
	    return null;
	}
	
	// get the collection or cluster description
	Element coll_about_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	
	Element coll_about_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE,  coll_name, page.getAttribute(GSXML.LANG_ATT));
	coll_about_message.appendChild(coll_about_request);
	
	Element coll_about_response = (Element)mr_.process(coll_about_message);

	// add the response to the page 
	Element response = (Element) GSXML.getChildByTagName(coll_about_response, GSXML.RESPONSE_ELEM);
	Element page_response = GSXML.duplicateWithNewName(doc_, response, "pageResponse", false);
	page.appendChild(page_response);

	// process using the stylesheet
	String stylesheet = GSFile.stylesheetFile(config_, "about.xsl");
	Document style_doc = converter_.getDOM(new File(stylesheet));
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	return (Element)transformer_.transform(style_doc, page);    

	
    }

    /** if we dont know the page type, use this method */
    protected Element unknownPage(Element page, Element request) {

	String page_name = request.getAttribute(GSXML.SUBACTION_ATT);

	String stylesheet = GSFile.stylesheetFile(config_, page_name+".xsl");
	if (stylesheet==null) {
	    // return the home page instead
	    return homePage(page, request);
	}

	// extract the params from the cgi-request, 
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_paramList, false);
	
	String coll_name = (String)params.get(GSCGI.COLLECTION_ARG);
	if (coll_name == null || coll_name.equals("")) {
	    // just process an empty page using the stylesheet
	    Element page_response = doc_.createElement(GSXML.PAGE_RESPONSE_ELEM);
	    page.appendChild(page_response);
	
	    Document style_doc = converter_.getDOM(new File(stylesheet));
	    GSXSLT.absoluteIncludePaths(style_doc, config_);
	    return (Element)transformer_.transform(style_doc, page);    
	    
	}
	// else get the coll description - actually this is the same as for the about page - should we merge these two methods??
	
	// if there is a service specified should we get the service description instead??
	// get the collection or cluster description
	Element coll_about_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	
	Element coll_about_request = GSXML.createBasicRequest(doc_, GSXML.REQUEST_TYPE_DESCRIBE,  coll_name, page.getAttribute(GSXML.LANG_ATT));
	coll_about_message.appendChild(coll_about_request);
	
	Element coll_about_response = (Element)mr_.process(coll_about_message);

	// add the response to the page 
	Element response = (Element) GSXML.getChildByTagName(coll_about_response, GSXML.RESPONSE_ELEM);
	Element page_response = GSXML.duplicateWithNewName(doc_, response, "pageResponse", false);
	page.appendChild(page_response);


	Document style_doc = converter_.getDOM(new File(stylesheet));
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	return (Element)transformer_.transform(style_doc, page);    
	
    }
}
